'****************************************************************
'*  Name    : Watchdog_DebounceB.inc                            *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 14/05/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Debounce 8 bits at a time                         *
'*          : New special watchdog, Timinmg 1 mS                *
'*  Code powered by the Proton Development Suite Version 3583   *
'****************************************************************
'
'
'===============================================================================
' This program is free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es un software libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' aprci.
'===============================================================================
'
' Code Debounce:
' Based on Scott Dattalo's code.
' What is it and what is used for? Well, if you ever looked at the 
' current through (or the voltage across) a mechanical switch or a relay as it was 
' changing states you would notice that the transition is not always a nice, clean 
' step function.
' Often there are several changes before the switch makes its transition.
' In this case, the switch is said to be "bouncing". However, the same 
' behavior can be observed in dirty switches or switches that are switching a 
' heavy load.
' Some times the bouncing can be removed with mild (a good vague term) 
' analog filtering and a comparator with hysteresis. Unfortunately this won't work 
' for all cases. And even if it did, the extra cost in hardware may not be 
' justifiable; especially since you can do most of the work in software.
'
' Typical debouncing routines sample the switch's state at a fairly high rate 
' (vague term # 2). When a change in the state is detected, the routine will count 
' anywhere from 1 to 4 to 8 to whatever samples to make sure the transition was 
' not a glitch and that the switch has settled to its new state.
' If you have many switches that need to be debounced simultaneously,
' it gets rather unwieldy keeping track of the counters and etc. This is emperically
' demonstrated by the brute-force debounce routines you typically see. Here's a 
' routine that will simultaneously debounce 8 inputs and count 4 samples before
' declaring a switch has changed states. There are no loops or gotos.
' The eight 2-bit counters are arranged as "vertical counters". In other words,
' the low order bit of the eight counters occupy one byte in RAM while the high
' order one another.
' The counters are incremented (actually decremented) using Boolean logic.
' Different examples of the vertical counters are available on the vertical counters page.
'
' The purpose of this routine is to debounce, i.e. digitally low pass filter
' inputs. The algorithm handles upto 8 bits at a time. An input is considered
' filtered if it has not changed states in the last 4 samples.
'
' 2-bit cyclic vertical counters count the 4 samples. As long as there is no
' change, the counters are held in the reset state of 00b. When a change is detected
' between the current sample and the filtered or debounced sample, the counters
' are incremented. The counting sequence is 00,01,10,11,00... When the counters
' roll over from 11b to 00b, the debounced state is updated. If the input changes
' back to the filtered state while the counters are counting, then the counters
' are re-initialized to the reset state and the filtered state is unaffected.
' In other words, a glitch or transient input has been filtered.
'    
' The 2-bit counters are arranged "vertically". In other words 8 counters
' are formed with 2 bytes such that the corresponding bits in the bytes are
' paired (e.g. MSBit of each byte is paired to form one counter).
' The counting sequence is 0,1,2,3,0,1,... And the state tables and Karnaugh
' maps are:
'
' State Table:     Karnaugh Maps:
' pres  next      B
' SS  SS         0   1
' AB  AB       +---+---+    +---+---+
'--------   A 0|   | 1 |    | 1 |   |
' 00  01       +---+---+    +---+---+
' 01  10      1| 1 |   |    | 1 |   |
' 10  11       +---+---+    +---+---+
' 11  00      A+ = A ^ B     B+ = ~B
'
' Here's the PIC code that implements the counter:
'       MOVF    SB,W    ;W = B
'       XORWF   SA,F    ;A+ = A ^ B
'       COMF    SB,F    ;B+ = ~B
'
'-------------------------------------------------------------------------------
$ifndef _DebounceInt_inc_
$define _DebounceInt_inc_
'-------------------------------------------------------------------------------
    $if _core <> 16
    $error "This Library is for the PIC18 ONLY!"
    $endif
'-------------------------------------------------------------------------------
    $define FOSC Xtal
'===============================================================================
' Global variables for the digital filter.
'
    Dim _Input_Bits As Byte System          ' Inputs buffer
    Dim _CONT_A As Byte System              ' Vertical counter A
    Dim _CONT_B As Byte System              ' Vertical counter B

    $ifndef DebounceOut_Var
    $define DebounceOut_Var
    Dim DebounceOut As Byte System          ' Outputs filter buffer
    $endif

    Dim DebounceTiming As Byte System       ' Interrupts counter
    Dim DebounceTimingTrigg As Byte System  ' Number of interrupts choosen by the user.
    Dim DelayCounter As Word System
    
    Dim Watchdog_Debounce_Flags As Byte System
    Dim F_Delay As Watchdog_Debounce_Flags.0        ' Flag for the MyDelayMS(x) command.
    Dim WDT_Detected As Watchdog_Debounce_Flags.1    
        
' Global Variables for the edge detection.
'
    $ifdef _RisingEdgesDetect_
    Dim RisingEdgeInputNew As Byte System 
    Dim RisingEdgeInputOld As Byte System 
    $ifndef RisingEdgeDetectedBits_Var
    Dim RisingEdgeDetectedBits As Byte System
    $endif    
    $endif

    $ifdef _FallingEdgesDetect_
    Dim FallingEdgeInputNew As Byte System
    Dim FallingEdgeInputOld As Byte System
    $ifndef FallingEdgeDetectedBits_Var
    Dim FallingEdgeDetectedBits As Byte System
    $endif
    $endif
    
    ' All variables of the Debounce routine must to be in Bank0.
    $if _device = _18F25K20
    #if DebounceOut >= Bank0_End
        #error "Debounce Counters Macro variables must reside within bank 0" 
    #endif      
    $endif

    $if _device = _18F25K22
    #if DebounceOut >= BankA_End
        #error "Debounce Counters Macro variables must reside within bank 0" 
    #endif      
    $endif

    Dim DataWord As Word
    
'-------------------------------------------------------------------------------
' Addresses of eeprom data.
    ' Using the last 4 four bytes of the eeprom memory.
    $if (_eeprom > 0)
    Symbol ADD_WDT_COUNTER = _eeprom - 2                ' Word, Address of the Watchdog counter.
    Symbol ADD_WDT_COUNTER_Check = ADD_WDT_COUNTER - 2  ' Word, Address of the Watchdog check.
    $else
        $error "This PIC doesn't have an eeprom memory!"
    $endif
'*******************************************************************************
' Interrupt declare.
    On_Hardware_Interrupt GoTo High#Interrupt#Start
'    On_Low_Interrupt GoTo Low#Interrupt#Start
'*******************************************************************************
'*******************************************************************************
    GoTo Over#Interrupt#Handler   

'    Include "D:\MyIncFiles\VersionFile.Inc"
    Include "VersionFile.inc"
'===============================================================================
' High Interrupt Handler for the debounce library command.
' The Timer2 is set for making interrupts every 4,0 mS.
' The additional counter (DebounceTiming) allows to read the inputs following the DebounceTimingTrigg value.
'
    Declare Reminders = OFF
High#Interrupt#Start:
    Context Save
'-------------------------------------------------------------------------------
    ' Debounce routine handler.
    If F_TMR2IF = 1 Then            ' Check if this is a timer2 interrupt?
        F_TMR2IF = 0                ' Clear the Timer2 interrupt flag for next time.
        $ifdef _DebugInterrupt_
        Set OutTest
        $endif
        Inc DebounceTiming
        If DebounceTiming = DebounceTimingTrigg Then
            Declare Warnings = OFF
            GoSub SetInputsToDigital_Sub                                ' Set all input used pins as digital
            Declare Warnings = On
            $ifdef InputPORT
                ' The TRIS is set before reading because it is more secure againts very high noise.
                InputPORTTRIS = InputPORTTRIS & InputPORTTRIS_InvertValue   ' Clear all input bits
                InputPORTTRIS = InputPORTTRIS | InputPORTTRIS_Value         ' Set TRIS the bits of InputPORT
                _Input_Bits = InputPORT                                     ' Read the InputPORT bits To the virtual Input PORT. 
            $endif 
 
            ' Copy all individual pins from the real hardware inputs to the virtual input PORT.
            $ifdef InputPin0
            Input InputPin0
            Set _Input_Bits.0
            If InputPin0 = 0 Then
                Clear _Input_Bits.0
            EndIf
            $endif
            $ifdef InputPin1
            Input InputPin1
            Set _Input_Bits.1
            If InputPin1 = 0 Then
                Clear _Input_Bits.1
            EndIf
            $endif
            $ifdef InputPin2
            Input InputPin2
            Set _Input_Bits.2
            If InputPin2 = 0 Then
                Clear _Input_Bits.2
            EndIf
            $endif            
            $ifdef InputPin3
            Input InputPin3
            Set _Input_Bits.3
            If InputPin3 = 0 Then
                Clear _Input_Bits.3
            EndIf
            $endif            
            $ifdef InputPin4
            Input InputPin4
            Set _Input_Bits.4
            If InputPin4 = 0 Then
                Clear _Input_Bits.4
            EndIf
            $endif
            $ifdef InputPin5
            Input InputPin5
            Set _Input_Bits.5
            If InputPin5 = 0 Then
                Clear _Input_Bits.5
            EndIf
            $endif
            $ifdef InputPin6
            Input InputPin6
            Set _Input_Bits.6
            If InputPin6 = 0 Then
                Clear _Input_Bits.6
            EndIf
            $endif
            $ifdef InputPin7
            Input InputPin7
            Set _Input_Bits.7
            If InputPin7 = 0 Then
                Clear _Input_Bits.7
            EndIf
            $endif

            ' The debounce asm routine, 8 bits at the same time.
            Asm
            ;The current sample is _Input_Bits            
            ;Increment the vertical counter
            Movf    _CONT_B,W,0
            Xorwf   _CONT_A,F,0
            Comf    _CONT_B,F,0
            ;See if any changes occurred 
            Movf    _Input_Bits,W,0
            Xorwf   DebounceOut,W,0
            ;Reset the counter if no change has occurred
            Andwf   _CONT_B,F,0
            Andwf   _CONT_A,F,0
            ;Determine the counter's state 
            Movf    _CONT_B,W,0
            Iorwf   _CONT_A,W,0
            ;Clear all bits that are filtered-or more accurately, save
            ;the state of those that are being filtered
            Andwf   DebounceOut,F,0
            Xorlw   0xFF
            ;Re-write the bits that haven't changed.
            Andwf   _Input_Bits,W,0
            Iorwf   DebounceOut,F,0
            ;the 8 filtered bits are in DebounceOut Variable
            EndAsm
            DebounceTiming = 0                                          ' Clear the interrupts counter.

            ' The code is written in strange manner (with WREG), because it is very closed to assembler
            ' to have less code as possible. It is very efficient.
            ' The code write the events only. It is impossible to clear any event by this code.
            ' The events must to be clear (bit by bit) after read by the user in the main code.
            $ifdef _FallingEdgesDetect_
            ' Detect input bits that change from 1 to 0 - falling edge
            WREG = DebounceOut & FallingEdgeMask                        ' Take care of the defined bits only and read the inputs.
            FallingEdgeInputNew = WREG                                  ' Save this value as new
            WREG = FallingEdgeInputOld ^ WREG                           ' Compare with the old bits
            WREG = FallingEdgeInputOld & WREG                           ' Confirm all bits have changed.
            FallingEdgeDetectedBits = FallingEdgeDetectedBits | WREG    ' Write only the results bits in the buffer. 
            FallingEdgeInputOld = FallingEdgeInputNew                   ' Save the new value for the next read.
            $endif
            $ifdef _RisingEdgesDetect_
            ' Detect input bits that change from 0 to 1 - rising edge
            WREG = DebounceOut & RisingEdgeMask                         ' Take care of the defined bits only and read the inputs.
            RisingEdgeInputNew = WREG                                   ' Save this value as new
            WREG = RisingEdgeInputOld ^ WREG                            ' Compare with the old bits
            WREG = RisingEdgeInputNew & WREG                            ' Confirm all bits have changed.
            RisingEdgeDetectedBits = RisingEdgeDetectedBits | WREG      ' Write only the results bits in the buffer. 
            RisingEdgeInputOld = RisingEdgeInputNew                     ' Save the new value for the next read.
            $endif
        EndIf
        '-----------------------------------------------------------------------
        ' New code for the MyDelay 1mS.
        If F_Delay = 1 Then             ' The Delay is started.
            $ifndef _TestWatchDogReset_
            $ifdef WatchDog_On
            Clrwdt
            $endif
            $endif  
            If DelayCounter = 0 Then    ' Check if delay counter reaches 0.
                F_Delay = 0             ' Delay OFF, clear the delay FLAG.
            EndIf
            Dec DelayCounter            ' Decrements the DelayCounter every 1mS.
        EndIf  
        '-----------------------------------------------------------------------
        $ifdef _DebugInterrupt_
        Clear OutTest
        $endif
    EndIf
    'END OF DEBOUNCE ROUTINE HANDLER.     
'-------------------------------------------------------------------------------
' Here another user interrupt code....
' .......
' .......
' .......
'-------------------------------------------------------------------------------
END_INT:
'-------------------------------------------------------------------------------
    Context Restore                     ' Exit the interrupt handler and restore variables
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
' Low Interrupt Handler: 
'Low#Interrupt#Start:
'    Context Save
'    Nop
'    Context Restore                     ' Exit the interrupt handler and restore variables
'-------------------------------------------------------------------------------
    Declare Reminders = On
'===============================================================================
' Some macro for the debounce library.
'
' Input the Reading Timing value in mS.
' The value must To be a multiple of 1mS. Example: 5 to 255 mS.
'
$define InitDebounceRate(pTiming) _mInitDebounce pTiming
    
_mInitDebounce Macro- pTiming
	#if (Prm_Count != 1)         
		#error "Incorrect amount of parameters for _mDebounceInit macro. 1 Parameter required!"	
		Exitm
    #else
        #if (Prm_1 != Num8)      
            #error "_mDebounceInit Macro - (Param1) Should be a number8"
            Exitm
        #else    
            $ifdef WatchDog_On
            Clrwdt
            $endif 
            #if (pTiming < 5)
            DebounceTimingTrigg = 5
            #else
            Num_Byte pTiming, DebounceTimingTrigg
            #endif
            F_TMR2IE = 1                    ' Enable the Timer2 interrupt 
            F_TMR2ON = 1                    ' Starts the Timer2. (T2CON)
            INTCON = INTCON | %11000000     ' Enable all interrupts.        
        #endif
    #endif        
Endm
'-------------------------------------------------------------------------------
' Make a Milisegond Delay:
$define MyDelayMS(pDelay) '
    DelayCounter = pDelay '
    GoSub MYDELAY_Sub

MYDELAY_Sub:
    F_Delay = 1                 ' Starts the delay. (enabled the interrupt counter module).
    While F_Delay = 1 : Wend    ' Waiting the delay flag cleared by the interrupt timer.
    Return

'------------------------------------------------------------------------------- 
' Clear the Watchdog LED. (Delay 5 MS)
    $define ClearWDTCounter()         '
    WDT_LED = 0                       '
    DataWord = 0                      '
    EWrite ADD_WDT_COUNTER,[DataWord]

'-------------------------------------------------------------------------------
' Set every input pin to digital if the InputAnselPinx/PORTx has been declared.
SetInputsToDigital_Sub:
    $ifdef InputAnselPORT
    InputAnselPORT = 0
    $endif
    $ifdef InputAnselPin0
    InputAnselPin0 = 0
    $endif
    $ifdef InputAnselPin1
    InputAnselPin1 = 0
    $endif
    $ifdef InputAnselPin2
    InputAnselPin2 = 0
    $endif
    $ifdef InputAnselPin3
    InputAnselPin3 = 0
    $endif
    $ifdef InputAnselPin4
    InputAnselPin4 = 0
    $endif    
    $ifdef InputAnselPin5
    InputAnselPin5 = 0
    $endif    
    $ifdef InputAnselPin6
    InputAnselPin6 = 0
    $endif
    $ifdef InputAnselPin7
    InputAnselPin7 = 0
    $endif
    $ifdef InputPin0
    Input InputPin0
    $endif    
    $ifdef InputPin1
    Input InputPin1
    $endif 
    $ifdef InputPin2
    Input InputPin2
    $endif 
    $ifdef InputPin3
    Input InputPin3
    $endif    
    $ifdef InputPin4
    Input InputPin4
    $endif 
    $ifdef InputPin5
    Input InputPin5
    $endif     
    $ifdef InputPin6
    Input InputPin6
    $endif    
    $ifdef InputPin7
    Input InputPin7
    $endif 

    $ifdef InputPORTTRIS
    ' This code takes care of the TRIS values of other pins
    InputPORTTRIS = InputPORTTRIS & InputPORTTRIS_InvertValue   ' Clear all input bits
    InputPORTTRIS = InputPORTTRIS | InputPORTTRIS_Value         ' Set TRIS the bits of InputPORT
    $endif
    Return
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
' This String is compiled and used only if the 'Include "VersionFile.inc"' is written 
' at the top of your main program. Then this sentence is sent to the Terminal of the Amicus18.
$ifdef _HRSOutVersionFile_inc_
_Watchdog_Debounce_inc_Version_: CData "Watchdog_DebounceB.inc Version 1.0 - 14/05/2016",0
$endif
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
Over#Interrupt#Handler:
'*******************************************************************************
'*******************************************************************************
' STARTCODE HERE:
' Initializing some parameters.
'-------------------------------------------------------------------------------
    ' Testing the Watchdog flag and counter.
    $ifdef WatchDog_On
    ' At power up for first time the value of the Watchdog counter in the eeprom is $FFFF. It must be equal to 0.
    DataWord = ERead ADD_WDT_COUNTER_Check
    If DataWord <> $A5A5 Then
        DataWord = 0
        EWrite ADD_WDT_COUNTER,[DataWord]
        DataWord = $A5A5
        EWrite ADD_WDT_COUNTER_Check,[DataWord]
    EndIf

    ' Check the Watchdog Time-out Flag bit
    WDT_Detected = 0                        ' Clear the WDT flag first.
    If F_TO = 0 Then                        ' Try to read a watchdog reset occured.
        WDT_Detected = 1                    ' A wachtdog reset has occured.
        DataWord = ERead ADD_WDT_COUNTER    ' Read the value of the Watchdog counter saved in the eeprom.
        Inc DataWord                        ' Increment the watchdog counter
        EWrite ADD_WDT_COUNTER,[DataWord]   ' Save the new value in the eeprom (5mS delay to write)
    EndIf
    $endif
'-------------------------------------------------------------------------------
    ' Enable of disabled the watchdog
    ' The Watchdog is different with the PIC18FxxK22 series.
    $ifdef WatchDog_On
    WDTCON.0 = 1            ' WDT is turned ON
    $else
    WDTCON.0 = 0            ' WDT is turned OFF (with config fuses OFF) 
    $endif

    INTCON = 0              ' Disable all interrupts

    $ifdef WatchDog_On
    Clrwdt
    ' Set the WDT LED if the Watchdog reset is detected.
    DataWord = ERead ADD_WDT_COUNTER        ' Read the value of the Watchdog counter saved in the eeprom.
    If DataWord > 0 Then
        WDT_LED = 1                         ' Set the Watchdog LED.
    Else
        WDT_LED = 0                         ' Clear the Watchdog LED.    
    EndIf

    If WDT_Detected = 1 Then
        HRSOut "WATCHDOG RESET DETECTED", CR
        DataWord = ERead ADD_WDT_COUNTER                    ' Read the value of the Watchdog counter saved in the eeprom.
        HRSOut "WATCHDOG COUNTER: ", Dec DataWord, CR, CR
    EndIf
    $endif
'-------------------------------------------------------------------------------
    PIR1 = 0                ' Peripheral Interrupt Request
    PIE1 = 0                ' Peripheral Interrupt Enable
    IPR1 = 0                ' Peripheral Interrupt Priority Register 1
'===============================================================================
' Initializing the timer2 & PR2 to generate the differents timings.
' For others timing copy the structure and modify the values of T2CON & PR2.
'-------------------------------------------------------------------------------
' Same structure for 18F2xk22 & 18F2xk20 & 18F452 series.
    #if (FOSC == 4)
        ' TMR2 8Bits counter, Postscaler 1/3, Prescaler 1/1, Osc Off, Clock FOSC/4 (FOSC: Xtal 4 Mhz)
        ' For Timer2 interrupt every 1,00 mS 
        T2CON = %00011000
        PR2 = 250
    #endif
    #if (FOSC == 8)
        ' TMR2 8Bits counter, Postscaler 1/7, Prescaler 1/1, Osc Off, Clock FOSC/4 (FOSC: Xtal 8 Mhz)
        ' For Timer2 interrupt every 1,00 mS 
'        T2CON = %00111000
        T2CON = %00111000        
        PR2 = 250
    #endif
    #if (FOSC == 16)
        ' TMR2 8Bits counter, Postscaler 1/1, Prescaler 1/4, Osc Off, Clock FOSC/4 (FOSC: Xtal 4 + PLL = 16 Mhz) or (FOSC: Xtal 16Mhz)
        ' For Timer2 interrupt every 1,00 mS 
        T2CON = %00011001
        PR2 = 250
    #endif
    #if (FOSC == 20)
        ' TMR2 8Bits counter, Postscaler 1/4, Prescaler 1/4, Osc Off, Clock FOSC/4 (FOSC: Xtal 20 Mhz)
        ' For Timer2 interrupt every 1,00 mS 
        T2CON = %00100001
        PR2 = 250
    #endif
    #if (FOSC == 40)
        ' TMR2 8Bits counter, Postscaler 1/9, Prescaler 1/4, Osc Off, Clock FOSC/4 (FOSC: Xtal 20 Mhz) or (FOSC: Xtal 10 Mhz + PLL)
        ' For Timer2 interrupt every 1,00 mS 
        T2CON = %01001001
        PR2 = 250
    #endif
    #if (FOSC == 64)
        ' TMR2 8Bits counter, Postscaler 1/16, Prescaler 1/4, Osc Off, Clock FOSC/4 (FOSC: Xtal 16 + PLL = 64Mhz)
        ' For Timer2 interrupt every 1,00 mS
        T2CON = %01111001
        PR2 = 250
        'PR2 = 248       ' 250 -2 = 248 to adjust the MyDelayMS(x) precision +0,25%
    #endif   
'===============================================================================
    DebounceTiming = 0                  ' Clear the Interrupts counter.
    DebounceOut = VirtualPORT_IdleState ' Load the virtual output PORT with the idle state of every pin.

    $ifdef _FallingEdgesDetect_
    ' Falling detection initialization
    WREG = VirtualPORT_IdleState & FallingEdgeMask
    FallingEdgeInputNew = WREG
    FallingEdgeInputOld = WREG
    FallingEdgeDetectedBits = 0
    $endif  
    $ifdef _RisingEdgesDetect_
    ' Rising detection initialization
    WREG = VirtualPORT_IdleState & RisingEdgeMask
    RisingEdgeInputNew = WREG
    RisingEdgeInputOld = WREG
    RisingEdgeDetectedBits = 0
    $endif

    ' Prepare the Timer2 interrups
    F_IPEN = 0                  ' RCON register. Disable priority levels On interrupts (PIC16CXXX Compatibility mode)
    F_TMR2IP = 1                ' High priority interrupt for Timer2 (in Case of...)
    F_TMR2IF = 0                ' Clear the Timer2 interrupt flag 
'*******************************************************************************
' Set every input pin to digital if the InputAnselPinx has been declared.
    GoSub SetInputsToDigital_Sub
'-------------------------------------------------------------------------------
' Set every output pin to digital if the InputAnselPinx has been declared.
    $ifdef OutAnselPin0
    OutAnselPin0 = 0
    $endif
    $ifdef OutAnselPin1
    OutAnselPin1 = 0
    $endif    
    $ifdef OutAnselPin2
    OutAnselPin2 = 0
    $endif    
    $ifdef OutAnselPin3
    OutAnselPin3 = 0
    $endif
    $ifdef OutAnselPin4
    OutAnselPin4 = 0
    $endif    
    $ifdef OutAnselPin5
    OutAnselPin5 = 0
    $endif
    $ifdef OutAnselPin6
    OutAnselPin6 = 0
    $endif    
    $ifdef OutAnselPin7
    OutAnselPin7 = 0
    $endif
    $ifdef OutAnselPin8
    OutAnselPin8 = 0
    $endif

    $ifdef OutPORT1TRIS
    OutPORT1TRIS = OutPORT1TRIS & OutPORT1TRIS_Value
    $endif    
    $ifdef OutPORT2TRIS
    OutPORT2TRIS = OutPORT2TRIS & OutPORT2TRIS_Value
    $endif    
    $ifdef OutPORT3TRIS
    OutPORT3TRIS = OutPORT3TRIS & OutPORT3TRIS_Value
    $endif  
'===============================================================================
$endif      ' _DebounceInt_inc_
'-------------------------------------------------------------------------------

